@@ -2,7 +2,7 @@
 package MRTG_lib;
 
 ###################################################################
-# MRTG 2.10.13  Support library MRTG_lib.pm
+# MRTG 2.10.14  Support library MRTG_lib.pm
 ###################################################################
 # Created by Tobias Oetiker <oetiker@ee.ethz.ch>
 #            and Dave Rand <dlr@bungi.com>
@@ -17,8 +17,9 @@ package MRTG_lib;
 
 require 5.005;
 use strict;
-use SNMP_util "0.97";
-use vars qw($OS $SL $PS @EXPORT @ISA $VERSION %mrtgrules);
+use SNMP_util "1.03";
+use vars qw($OS $SL $PS @EXPORT @ISA $VERSION %timestrpospattern);
+my %mrtgrules;
 
 BEGIN {
     # Automatic OS detection ... do NOT touch
@@ -51,6 +52,14 @@ require Exporter;
 
 $VERSION = 2.100001;
 
+%timestrpospattern =
+      (
+       'NO' => 0,
+       'LU' => 1,
+       'RU' => 2,
+       'LL' => 3,
+       'RL' => 4
+      );
 
 %mrtgrules =
       (                         # General CFG
@@ -308,7 +317,13 @@ $VERSION = 2.100001;
        [sub{$_[0] && (-e $_[0])}, sub{"Threshold program $_[0] cannot be executed"}],
 
        'threshprogoko[]' =>
-       [sub{$_[0] && (-e $_[0])}, sub{"Threshold program $_[0] cannot be executed"}]
+       [sub{$_[0] && (-e $_[0])}, sub{"Threshold program $_[0] cannot be executed"}],
+
+       'timestrpos[]' => 
+       [sub{$_[0] =~ /^(no|[lr][ul])$/i}, sub{"Must be a string of NO, LU, RU, LL, RL"}],
+
+       'timestrfmt[]' => 
+       [sub{1}, sub{"Internal Error"}] #what ever the user chooses.
 );
 
 # config file reading
@@ -908,6 +923,12 @@ sub cfgcheck ($$$$) {
         if (! defined $$rcfg{'yscale'}{$rou}) {
             $$rcfg{'yscale'}{$rou} = 1.0;
         }
+        if (! defined $$rcfg{'timestrpos'}{$rou}) {
+            $$rcfg{'timestrpos'}{$rou} = 'NO';
+        }
+        if (! defined $$rcfg{'timestrfmt'}{$rou}) {
+            $$rcfg{'timestrfmt'}{$rou} = "%Y-%m-%d %H:%M";
+        }
         if ($error eq "yes") {
             die "ERROR: Please fix the error(s) in your config file\n";
         }
@@ -1410,7 +1431,7 @@ sub writeconfcache ($$) {
         @hosts = @{$$confcache{___updated}} ;
         delete $$confcache{___updated};
     } else {
-        @hosts = keys %{$confcache}
+        @hosts = grep !/^___/, keys %{$confcache}
     }
     foreach my $host (sort @hosts) {
         foreach my $method (sort keys %{$$confcache{$host}}) {
@@ -1450,7 +1471,7 @@ sub populateconfcache ($$$$$) {
     my $reread = shift;
     my $snmpoptions = shift || {};
     return if defined $$confcache{$host} and not $reread;
-
+    
     my $snmp_errlevel = $SNMP_Session::suppress_warnings;
     $SNMP_Session::suppress_warnings = 3;    
 
@@ -1466,6 +1487,7 @@ sub populateconfcache ($$$$$) {
     # it seems that some devices only give back sensible data if their tables
     # are walked in the right ordere ....
     foreach my $node (@nodes) {
+	next if $confcache->{___deadhosts}{$host} and time - $confcache->{___deadhosts}{$host} < 300;
 	$SNMP_Session::errmsg = undef;
 	@ret = snmpwalk(v4onlyifnecessary($host, $ipv4only), $snmpoptions, $node);
 	unless ( $SNMP_Session::errmsg){
@@ -1481,9 +1503,15 @@ sub populateconfcache ($$$$$) {
 		  }
 	      };
 	} else {
+  	    $confcache->{___deadhosts}{$host} = time
+		if $SNMP_Session::errmsg =~ /no response received/;
 	    debug('snpo',"Skipping $node scanning because $host does not seem to support it");
 	}
     }
+    if ($confcache->{___deadhosts}{$host} and time - $confcache->{___deadhosts}{$host} < 300){
+	$SNMP_Session::suppress_warnings = $snmp_errlevel;
+	return;
+    }
     $SNMP_Session::errmsg = undef;
     @ret = snmpwalk(v4onlyifnecessary($host, $ipv4only), $snmpoptions, "ifPhysAddress");
     unless ( $SNMP_Session::errmsg){
