@@ -1,5 +1,5 @@
 /*
- MRTG 2.10.13  -- Rateup
+ MRTG 2.10.14  -- Rateup
  *********************
 
  Rateup is a fast add-on to the great MRTG Traffic monitor.  It makes
@@ -81,7 +81,7 @@
 #include <gd.h>
 #include <gdfonts.h>
 
-char *VERSION = "2.10.13";
+char *VERSION = "2.10.14";
 char *program,*router,*routerpath;
 int histvalid;
 
@@ -228,7 +228,7 @@ for debuging
   }
 
  
-static void image(file,maxvi,maxvo,maxx,maxy,xscale,yscale,growright,step, bits, ytics, yticsf, peak)
+static void image(file,maxvi,maxvo,maxx,maxy,xscale,yscale,growright,step, bits, ytics, yticsf, peak, currdatetimeformat, currdatetimepos)
 char *file;
 long long  maxvi,maxvo;
 long maxx;
@@ -237,6 +237,8 @@ double xscale,yscale;
 int ytics; /* number of tics on the y axis*/
 double yticsf; /* scale everything on the y axis with this factor*/
 int peak;
+char *currdatetimeformat;
+int currdatetimepos;
 {
     FILE *fo;
     int i,x,y,n,type;
@@ -259,6 +261,15 @@ int peak;
     double sca_max_q,dummy;
     double percent;
     char *short_si_out;
+    char currdatetimestr[256];
+    time_t currdatetime;
+    int currdatetimepos_x, currdatetimepos_y;
+
+#define NO_TIMESTAMPSTR (0)
+#define LU_CORNER (1)
+#define RU_CORNER (2)
+#define LL_CORNER (3)
+#define RL_CORNER (4)
     
 
     struct HISTORY *lhist;
@@ -439,7 +450,7 @@ int peak;
         short_si_out = short_si[min((signed) (digits/3), kMGnumber)];
      }
       
-    if (maxv_q) {
+    if (maxv_q*yticsf >= 1) {
 /*    	digits = log10((double)maxv_q); */
     	digits = log10((double)maxv_q*yticsf);
     } else {
@@ -807,6 +818,31 @@ int peak;
     gdImageLine(graph, xtr(-3),ytr(0),xtr(-3),ytr(0), i_major);
    }
 
+    if (currdatetimepos > NO_TIMESTAMPSTR) {
+      currdatetime=time(NULL);
+      strftime(currdatetimestr,250,currdatetimeformat,localtime(&currdatetime));
+      switch ( currdatetimepos ) {
+        case LL_CORNER:
+          currdatetimepos_x = 3;
+          currdatetimepos_y = YSIZE - gdFontSmall->h - 3;
+          break;
+        case RL_CORNER:
+          currdatetimepos_x = XSIZE - strlen(currdatetimestr) * gdFontSmall->w - 3;
+          currdatetimepos_y = YSIZE - gdFontSmall->h - 3;
+          break;
+        case LU_CORNER :
+          currdatetimepos_x = 3;
+          currdatetimepos_y = 1;
+          break;
+        case RU_CORNER:
+        default:
+          currdatetimepos_x = XSIZE - strlen(currdatetimestr) * gdFontSmall->w - 3;
+          currdatetimepos_y = 1;
+      };
+      gdImageString(graph, gdFontSmall,
+          currdatetimepos_x, currdatetimepos_y,
+          currdatetimestr, i_grid);
+    }
 
     if ((fo = fopen(file,"wb")) == NULL) {
       perror(program);
@@ -1388,8 +1424,10 @@ char **argv;
 		    atol(argv[argi+10]), /* bits */
 		    atol(argv[argi+11]), /* ytics */
 		    atof(argv[argi+12]), /* yticsfactor */
-		    0);
-	      used = 13;
+		    0,
+		    argv[argi+13],
+		    atol(argv[argi+14]));
+	      used = 15;
 	      break;
 	    case 'p':	/* Create PPM Image record with Peak values*/
 		image(argv[argi+1],
@@ -1404,8 +1442,10 @@ char **argv;
 	   	      atol(argv[argi+10]), /* bits */
 		      atol(argv[argi+11]), /* ytics */
 		      atof(argv[argi+12]), /* yticsfactor */
-		      1);
-		used = 13;
+		      1,
+		      argv[argi+13],
+		      atol(argv[argi+14]));
+		used = 15;
 		break;
 	    case 'r':	/* Create random records, then update */
 		for (x=0; x<histvalid; x++) {
